/*
 **************************************************************************
 *
 * Boot-ROM-Code to load an operating system across a TCP/IP network.
 *
 * Module:  getext.c
 * Purpose: Get BOOTP vendor extensions file
 * Entries: get_ext
 *
 **************************************************************************
 *
 * Copyright (C) 1995-1999 Gero Kuhlmann <gero@gkminix.han.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **************************************************************************
 *
 * We can print error messages within this module as it gets never called
 * by a PXE function.
 */


#include <general.h>
#include <kernel/net.h>
#include <kernel/arpa.h>
#include <kernel/romlib.h>
#include <pxe/common.h>
#include "arpapriv.h"
#include "bootp.h"



/*
 **************************************************************************
 * 
 * Get vendor extensions file with TFTP.
 */
int get_ext(ftag)
unsigned char *ftag;
{
  register unsigned char *cp;
  t_tftp_read_file params;
  int ret, i;

  /* Setup parameters structure */
  printf("BOOTP: Loading %ls: ", ftag + 1, *ftag);
  memset(&params, 0, sizeof(params));
  if ((i = *ftag) > sizeof(params.filename))
	i = sizeof(params.filename);
  memcpy(params.filename, ftag + 1, i);
  params.server = boot_rec.bp_siaddr;
  params.gateway = n_IP_ANY;
  if ((cp = get_vend(VEND_ROUTER)) != NULL)
	params.gateway = *((t_ipaddr *)(cp + 1));
  if ((cp = get_vend(VEND_END)) == NULL) {
	printf("Missing END tag\n");
	ret = PXENV_STATUS_FAILURE;
	goto ext_ret;
  }
  cp--;
  params.buffer = far2long(MK_FP(getds(), cp));
  params.bufsize = (unsigned char *)(&boot_rec) + sizeof(boot_rec) - cp;

  /* Now read the file via TFTP */
  if ((ret = tftp_read_file(&params)) != PXENV_STATUS_SUCCESS) {
	printf("TFTP error %x\n", ret);
	goto ext_ret;
  }

  /*
   * Check that the file starts with the correct vendor extension. If it
   * does, overwrite the signature with all NOPs so that the record
   * becomes valid.
   */
  ret = PXENV_STATUS_FAILURE;
  if (params.bufsize < (VM_SIZE + 1) ||
      *((unsigned long *)&(boot_rec.bp_vend[0])) != VM_RFC1048) {
	printf("Invalid file\n");
	goto ext_ret;
  }
  for (i = 0; i < VM_SIZE; i++)
	*(cp++) = VEND_NOP;

  /* Finally check if we have an end identifier at all */
  if (get_vend(VEND_END) == NULL) {
	printf("No end tag\n");
	goto ext_ret;
  }
  ret = PXENV_STATUS_SUCCESS;
  printf("ok\n");

ext_ret:
  return(ret == PXENV_STATUS_SUCCESS);
}

