/*
 **************************************************************************
 *
 * Boot-ROM-Code to load an operating system across a TCP/IP network.
 *
 * Module:  getvend.c
 * Purpose: Get information out of BOOTP/DHCP answer
 * Entries: get_vend
 *
 **************************************************************************
 *
 * Copyright (C) 1998,1999 Gero Kuhlmann <gero@gkminix.han.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include <general.h>
#include <kernel/net.h>
#include <kernel/arpa.h>
#include <kernel/romlib.h>
#include "bootp.h"



/*
 **************************************************************************
 * 
 * Find a tag in the BOOTP/DHCP record.
 *
 */
static unsigned char *find_tag(id, startp, endp)
int id;
unsigned char *startp;
unsigned char *endp;
{
  register unsigned char *cp = startp;

  while (*cp != VEND_END && cp < endp) {
	if (*cp != VEND_NOP) {
		if (*cp == id)
			break;
		cp += *(cp + 1) + 1;
	}
	cp++;
  }
  return(*cp == id ? cp + 1 : NULL);
}



/*
 **************************************************************************
 * 
 * Find a tag either in the vendor information area or the boot filename
 * or server name fields.
 */
static unsigned char *search_tag(id, startp, endp)
int id;
unsigned char *startp;
unsigned char *endp;
{
  register unsigned char *cp = startp;
  unsigned char *retp;

  if ((retp = find_tag(id, cp, endp)) == NULL &&
      (cp = find_tag(VEND_OVERLOAD, cp, endp)) != NULL) {
	cp++;
	if (*cp & VEND_OVR_FILE)
		retp = find_tag(id, boot_rec.bp_file,
					boot_rec.bp_file + BOOTP_FILE_SIZE);
	if (retp == NULL && (*cp & VEND_OVR_SNAME)) {
		retp = find_tag(id, boot_rec.bp_sname,
					boot_rec.bp_sname + BOOTP_SNAME_SIZE);
	}
  }
  return(retp);
}



/*
 **************************************************************************
 * 
 * Check if we have PXE vendor extensions
 */
static unsigned char *check_pxe(startp, endp)
unsigned char *startp;
unsigned char *endp;
{
  register unsigned char *cp;

  cp = search_tag(VEND_CLASS, startp, endp);
  if (cp != NULL &&
      (*cp != VEND_PXE_STRLEN ||
       memcmp(cp + 1, VEND_PXE_STRING, VEND_PXE_STRLEN)))
	cp = NULL;
  return(cp);
}



/*
 **************************************************************************
 * 
 * Return information from the vendor area of a BOOTP/DHCP record. Note
 * that this will work only with RFC1048-compliant messages.
 * It will return a pointer to the length byte of the tag.
 *
 */
unsigned char *get_vend(id)
int id;
{
#define endp ((unsigned char *)(&boot_rec) + sizeof(boot_rec))

  register unsigned char *cp = boot_rec.bp_vend;
  register int len;
  unsigned char *retp = NULL;

  if (*cp) {
	cp += VM_SIZE;
	if (id < VEND_PXE_BASE)
		retp = search_tag(id, cp, endp);
	else if (check_pxe(cp, endp) != NULL &&
	         (cp = search_tag(VEND_VENDOR, cp, endp)) != NULL)
		retp = find_tag(id & VEND_PXE_MASK, cp + 1, cp + *cp + 1);
  }
  return(retp);

#undef endp
}

