/*
 **************************************************************************
 *
 * Boot-ROM-Code to load an operating system across a TCP/IP network.
 *
 * Module:  netpriv.h
 * Purpose: Miscellaeous definitions for the network library
 * Entries: none
 *
 **************************************************************************
 *
 * Copyright (C) 1995-1999 Gero Kuhlmann <gero@gkminix.han.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _KERNEL_NET_NETPRIV_H
#define _KERNEL_NET_NETPRIV_H
#ifndef _USE_ASSEMBLER


/*
 **************************************************************************
 * 
 * Global variables in the network library:
 */
extern unsigned char myhwaddr[ETH_ALEN]; /* my own hardware address	*/
extern t_ipaddr      myipaddr;           /* my own IP address		*/
extern t_ipaddr      mynetmask;		 /* netmask for my network	*/



/*
 **************************************************************************
 * 
 * Default UDP timeout
 */
#define UDP_TIMEOUT	20		/* 20 seconds default timeout	*/



/*
 **************************************************************************
 *
 * Public routines of the UNDI module:
 */

/* Open UNDI */
extern int undi_open __P((void));

/* Close UNDI */
extern int undi_close __P((void));

/* Send packet using UNDI */
extern int undi_send __P((unsigned char *dest, int proto, int num,
					int sizes[], unsigned char bufs[]));

/* Get receive buffer from UNDI */
extern int undi_get_buf __P((int size, unsigned char *buf));



/*
 **************************************************************************
 *
 * External routines in the utility modules:
 */

/* Compute IP header checksum */
extern int ipchksum __P((unsigned char *buf, int len));


#endif				/* _USE_ASSEMBLER */
#endif				/* _KERNEL_NET_NETPRIV_H */

