!**************************************************************************
!*
!* Boot-ROM-Code to load an operating system across a TCP/IP network.
!*
!* Module:  undi.S
!* Purpose: Interface to the universal network driver interface
!* Entries: _undi_open, _undi_close, _undi_send, _undi_get_buf,
!*          net_start, net_stop
!*
!**************************************************************************
!*
!* Copyright (C) 1998,1999 Gero Kuhlmann <gero@gkminix.han.de>
!*
!*  This program is free software; you can redistribute it and/or modify
!*  it under the terms of the GNU General Public License as published by
!*  the Free Software Foundation; either version 2 of the License, or
!*  any later version.
!*
!*  This program is distributed in the hope that it will be useful,
!*  but WITHOUT ANY WARRANTY; without even the implied warranty of
!*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*  GNU General Public License for more details.
!*
!*  You should have received a copy of the GNU General Public License
!*  along with this program; if not, write to the Free Software
!*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
!*


!
!**************************************************************************
!
! Include assembler header files:
!
#include <general.inc>
#include <memory.inc>
#include <pxe/common.inc>
#include <pxe/undi.inc>
#include <hw/eth.inc>
#include <hw/i8259a.inc>


!
!**************************************************************************
!
! Size of interrupt stack:
!
STACK_SIZE	equ	1024


!
!**************************************************************************
!
! BSS segment.
!
	.bss

	extrn	_myhwaddr		! buffer holding current hardware addr

	.lcomm	bufaddr,4		! pointer to receive buffer
	.lcomm	bufsize,2		! size of data in receive buffer
	.lcomm	irqnum,2		! IRQ numbers for EOI
	.lcomm	irqvect,2		! offset to hardware IRQ vector
	.lcomm	oldhwint,4		! old hardware interrupt vector
	.lcomm	intstack,STACK_SIZE	! interrupt stack
	.lcomm	oldsp,4			! old stack pointer
	.lcomm	intcnt,2		! count interrupt requests
	.lcomm	undicnt,2		! count UNDI interrupt calls


!
!**************************************************************************
!
! Start code segment.
!
	.text

	public	net_start
	public	net_stop
	public	_undi_open		! define entry points
	public	_undi_close
	public	_undi_send
	public	_undi_get_buf

! External functions and data within text segment

	extrn	datseg
	extrn	callpxe
	extrn	_arp_poll
	extrn	_arp_recv
	extrn	_udp_recv


!
!**************************************************************************
!
! Startup the network driver interface.
! Input:  none
! Output: Carry set if error
! Registers changed: (E)AX, (E)BX, CX, SI, DI
!
net_start:

	push	bp

! Initialize the UNDI driver interface.

	sub	sp,#initialize_size
	mov	bp,sp
#ifdef IS386
	mov	dword ptr [bp + o_ui_protocolini],#0
#else
	mov	word ptr [bp + o_ui_protocolini + 0],#0
	mov	word ptr [bp + o_ui_protocolini + 2],#0
#endif
	mov	ax,#UNDI_INITIALIZE
	call	callpxe			! initialize UNDI interface
	add	sp,#initialize_size
	or	ax,ax			! check for error return
	jnz	start8

! Get information about the hardware which is serviced by the UNDI driver,
! like the current hardware address and the hardware interrupt vector.

	sub	sp,#getinfo_size
	mov	bp,sp
	mov	ax,#UNDI_GET_INFORMATION
	call	callpxe			! get UNDI driver information
	or	ax,ax			! check for error return
	jnz	start7

	mov	cx,[bp + o_ugi_hwaddrlen]
	cmp	cx,#ETH_ALEN		! check that the hardware address
	jne	start7			! length is correct

#ifdef IS386
if ETH_ALEN = 6
start0	equ	0
else
start0	equ	1
endif
#else
start0	equ	1
#endif

if start0 = 0
	mov	ebx,dword ptr [bp + o_ugi_currentaddr + 0]
	mov	dword ptr [_myhwaddr + 0],ebx
	mov	bx,word ptr [bp + o_ugi_currentaddr + 4]
	mov	word ptr [_myhwaddr + 4],bx
else
	cld
	push	ds
	mov	ax,ss
	mov	ds,ax
	lea	si,[bp + o_ugi_currentaddr]
	mov	di,#_myhwaddr		! copy current hardware address
	rep
	movsb
	pop	ds
endif

! Determine the hardware interrupt number and the address of the interrupt
! vector. We dont have to worry about interrupt 2 on AT and higher systems
! here. The UNDI driver will never return interrupt number 2 on those
! systems, and even if it does the BIOS will redirect interrupt 9 correctly
! and will take care of sending the EOI to the slave.

	mov	ax,[bp + o_ugi_intnumber]
	mov	si,ax
	mov	ah,#$FF
	cmp	al,#PIC_IRQ_NUM		! set interrupt number for master PIC
	jb	start1
	mov	ah,al			! set interrupt numbers for master and
	sub	ah,#PIC_IRQ_NUM		! slave PIC
	mov	al,#SLAVE_IRQ_NUM
	add	si,#MIN_SLAVE_HWINT - PIC_IRQ_NUM
	jmp	start2
start1:	add	si,#MIN_MASTER_HWINT
start2:	mov	[irqnum],ax		! save IRQ numbers
#ifdef IS186
	shl	si,#2
#else
	shl	si,#1
	shl	si,#1
#endif
	mov	[irqvect],si
	add	sp,#getinfo_size	! cleanup the stack

! Finally redirect the hardware interrupt vector

	push	es
	xor	ax,ax
	mov	es,ax
#ifdef IS386
	mov	ax,cs
	shl	eax,#16
	mov	ax,#hwint
	seg	es
	xchg	[si],eax
	mov	[oldhwint],eax
#else
	cli
	seg	es
	mov	ax,word ptr [si + 0]
	mov	word ptr [oldhwint + 0],ax
	seg	es
	mov	ax,word ptr [si + 2]
	mov	word ptr [oldhwint + 2],ax
	seg	es
	mov	word ptr [si + 0],#hwint
	seg	es
	mov	word ptr [si + 2],cs
	sti
#endif
	pop	es
	xor	ax,ax			! clear carry flag
	jmp	start9

start7:	add	sp,#getinfo_size
start8:	stc
start9:	pop	bp
	ret


!
!**************************************************************************
!
! Stop the network driver interface and restore the hardware interrupt
! vector.
! Input:  none
! Output: Carry set if error
! Registers changed: (E)AX, DX, SI
!
net_stop:

	push	bp
	push	es

! First restore the hardware interrupt vector. If the present vector is
! not our own we cant terminate.

	xor	ax,ax
	mov	es,ax
	mov	si,[irqvect]
#ifdef IS386
	mov	ax,cs
	shl	eax,#16
	mov	ax,#hwint		! check if current vector points to
	seg	es			! our int handler
	cmp	eax,[si]
	jne	stop8
	mov	eax,[oldhwint]
	seg	es			! restore old interrupt
	mov	[si],eax
#else
	cli
	seg	es
	mov	ax,word ptr [si + 0]
	seg	es
	mov	dx,word ptr [si + 2]
	sti
	cmp	ax,#hwint
	jne	stop8			! check if current vector points to
	mov	ax,cs			! our int handler
	cmp	dx,ax
	jne	stop8
	cli
	mov	ax,word ptr [oldhwint + 0]
	seg	es
	mov	[si + 0],ax		! restore old interrupt
	mov	ax,word ptr [oldhwint + 2]
	seg	es
	mov	[si + 2],ax
	sti
#endif

! Now shutdown the UNDI driver.

	sub	sp,#shutdown_size
	mov	bp,sp
	mov	ax,#UNDI_SHUTDOWN
	call	callpxe			! shutdown UNDI driver
	add	sp,#shutdown_size
	or	ax,ax			! check for error return
	jz	stop9
stop8:	stc
stop9:	pop	es
	pop	bp
	ret


!
!**************************************************************************
!
! Open a connection to the UNDI driver.
! Input:  none
! Output: error code (0 if no error)
!
_undi_open:

! Open the interface. Since we dont use multicast addresses, we
! can leave the multicast list empty.

	push	bp
	sub	sp,#open_size
	mov	bp,sp
	mov	word ptr [bp + o_uo_openflag],#0
	mov	word ptr [bp + o_uo_pktfilter],#UNDI_FLTR_DIRECTED + UNDI_FLTR_BROADCAST
	mov	word ptr [bp + o_uo_mcastbuf + o_uma_count],#0
	mov	ax,#UNDI_OPEN
	call	callpxe
	add	sp,#open_size
	pop	bp
	ret


!
!**************************************************************************
!
! Close connection to the UNDI driver.
! Input:  none
! Output: error code (0 if no error)
!
_undi_close:

	push	bp
	sub	sp,#close_size
	mov	bp,sp
	mov	ax,#UNDI_CLOSE
	call	callpxe
	add	sp,#close_size
	pop	bp
	ret


!
!**************************************************************************
!
! Send a packet
! Input:  1. arg  -  near pointer to destination address (0 if broadcast)
!         2. arg  -  protocol of outgoing packet
!         3. arg  -  number of send buffers
!         4. arg  -  near pointer to list of send buffer sizes
!         5. arg  -  near pointer to list of send buffer pointers
! Output: error code
!
_undi_send:

	penter	(transmit_size + tbd_size)	! setup standard stack frame

txbuf	equ	transmit_size
tbd	equ	transmit_size + tbd_size

! Check if we have to send anything at all

	xor	ax,ax
	getcarg	(cx,2)			! get number of send buffers
	jcxz	send9

! Setup the transmit protocol structure

	lea	bx,[bp - tbd]
	getcarg	(dx,1)			! get packet protocol
	getcarg	(ax,0)			! get pointer to destination address
	mov	dh,#UNDI_XMT_BROADCAST
	or	ax,ax			! is it a broadcast address?
	jz	send1
	mov	dh,#UNDI_XMT_DESTADDR
	mov	word ptr [bp - txbuf + o_ut_dest_off],ax
	mov	word ptr [bp - txbuf + o_ut_dest_seg],ds
send1:	mov	word ptr [bp - txbuf + o_ut_protocol],dx
	mov	word ptr [bp - txbuf + o_ut_tbd_off],bx
	mov	word ptr [bp - txbuf + o_ut_tbd_seg],ss

! Setup the transmit data block structure

	cld
	push	si
	push	di
	getcarg	(bx,3)			! get pointer to size list
	getcarg	(si,4)			! get pointer to buffer pointer list
	mov	di,#o_tbd_datablock
	mov	word ptr [bp - tbd + o_tbd_xmtlength],#0
	mov	word ptr [bp - tbd + o_tbd_blocks],cx
send2:	mov	dx,word ptr [bx]
	add	bx,#2
	mov	byte ptr [bp + di - tbd + o_tbd_tdptrtype],#1
	mov	word ptr [bp + di - tbd + o_tbd_tdlength],dx
#ifdef IS386
	lodsd
	mov	dword ptr [bp + di - tbd + o_tbd_tdaddr],eax
#else
	lodsw
	mov	word ptr [bp + di - tbd + o_tbd_tdaddr + 0],ax
	lodsw
	mov	word ptr [bp + di - tbd + o_tbd_tdaddr + 2],ax
#endif
	add	di,#datablk_size
	loop	send2
	pop	di
	pop	si

! Now finally call UNDI to send all packets

	push	bp
	lea	bp,[bp - txbuf]
	mov	ax,#UNDI_TRANSMIT
	call	callpxe
	pop	bp
send9:	pleave
	ret


!
!**************************************************************************
!
! Copy receive buffer into local buffer of higher protocol
! Input:  1. arg  -  maximum size of local buffer
!         2. arg  -  pointer to local buffer
! Output: number of bytes copied
!
_undi_get_buf:

	penter	(0)			! setup standard stack frame
	mov	cx,[bufsize]
	jcxz	getb9
	getcarg	(ax,0)			! get size of local buffer
	cmp	ax,cx
	jae	getb1			! make sure we dont overwrite the end
	mov	cx,ax			! of the local buffer
getb1:	push	ds
	push	es
	push	si
	push	di
	mov	bx,cx
	mov	ax,ds
	mov	es,ax
	getcarg	(di,1)			! get pointer to local buffer
	lds	si,[bufaddr]
	cld
	shr	cx,#1
	rep
	movsw				! copy receive buffer into local buffer
	jnc	getb2
	movsb
getb2:	mov	cx,bx
	pop	di
	pop	si
	pop	es
	pop	ds
	mov	word ptr [bufsize],#0
getb9:	mov	ax,cx
	pleave
	ret


!
!**************************************************************************
!
! Hardware interrupt routine.
! Input:  none
! Output: none
! Registers changed: none
!
hwint:

! This routine should never get called recursively because hardware
! interrupts should get turned off at the device level by the UNDI
! driver. However, if this doesnt work for some reason, we count the
! interrupt and return. The previous interrupt handler call will then
! recycle this routine at the end.

	cli
	push	ds
	seg	cs
	mov	ds,[datseg]
	inc	word ptr [intcnt]	! increment interrupt count
	cmp	word ptr [intcnt],#1	! check if this is the first call
	jne	hwint9

	mov	word ptr [oldsp + 0],sp
	mov	word ptr [oldsp + 2],ss
	seg	cs
	mov	ss,[datseg]
	mov	sp,#intstack + STACK_SIZE - 2
	pushall
	push	es

! Call the UNDI driver to handle the interrupt, and check if its ours.

hwint1:	sub	sp,#isr_size
	mov	bp,sp
	mov	word ptr [bp + o_uisr_funcflag],#UNDI_ISR_IN_START
	mov	ax,#UNDI_ISR
	call	callpxe
	or	ax,ax
	mov	ax,#UNDI_ISR_OUT_NOT_OURS
	jnz	hwint2
	mov	ax,word ptr [bp + o_uisr_funcflag]
hwint2:	add	sp,#isr_size
	or	ax,ax			! check if interrupt is ours
	jnz	hwint5

! Send specific EOI since this interrupt was ours.

	mov	ax,[irqnum]
	cli
	cmp	ah,#$FF			! check if we have to care for
	je	hwint3			! a slave PIC
	xchg	al,ah
	add	al,#CMD_S_EOI
	out	SLAVE_CMD,al		! send specific EOI to slave
	xchg	al,ah
hwint3:	add	al,#CMD_S_EOI
	out	MASTER_CMD,al		! send specific EOI to master
	sti

! Now actually process the hardware interrupt by copying the received
! data into the higher level buffers. We can do this with interrupts
! enabled since they should be disabled on the hardware level. If we
! reentered the UNDI driver for some strange reason, process the
! missing interrupt requests at the next lower level. To indicate
! this, the UNDI call counter is used.

	mov	ax,ds
	mov	es,ax
	inc	word ptr [undicnt]
hwint4:	call	dohwint
	jc	hwint8
	dec	word ptr [undicnt]
	jnz	hwint4

! Terminate the hardware interrupt processing. If this routine got
! called recursively, recycle it to handle the previous request.

hwint8:	cli
	dec	word ptr [intcnt]
	jnz	hwint1
	pop	es
	popall
#ifdef IS386
	lss	sp,[oldsp]
#else
	mov	ss,word ptr [oldsp + 2]
	mov	sp,word ptr [oldsp + 0]
#endif
hwint9:	pop	ds
	iret

! If the interrupt was not ours we have to pass it down to the old interrupt
! handler because PCI cards can have shared interrupts. This may end invoking
! the BIOS interrupt handler which will turn off the interrupt at the PIC.
! Therefore, we have to reenable interrupts after return from the old handler.

hwint5:	mov	ax,word ptr [oldhwint + 2]	! check if an old handler is
	or	ax,ax				! available
	jz	hwint8

	sti
	pushf
	call	dword ptr [oldhwint]	! call old handler

	mov	dx,#MASTER_IMR
	mov	cx,[irqnum]		! get interrupt number
	cmp	ch,#$FF			! check if its on master PIC
	je	hwint6
	mov	dx,#SLAVE_IMR		! no, its on the slave
	mov	cl,ch
hwint6:	cli
	in	al,dx			! read interrupt mask register
	mov	bl,#$01
	shl	bl,cl			! compute interrupt mask
	test	al,bl			! check if it has been masked
	jz	hwint8
	not	bl
	and	al,bl			! unmask interrupt again
	out	dx,al
	jmp	hwint8


!
!**************************************************************************
!
! Actually process a hardware interrupt by calling UNDI repeatedly to
! deliver all received buffers.
! Input:  none
! Output: Carry flag set if UNDI called recursively
! Registers changed: (E)AX, BX, CX, DX, BP, SI, DI
!
dohwint:

#ifdef IS186
	push	#0
#else
	xor	ax,ax
	push	ax
#endif
	call	_udp_recv		! recycle UDP receive buffer
	sub	sp,#isr_size - 2
	mov	bp,sp
	mov	ax,#UNDI_ISR_IN_PROCESS
dohw1:	mov	[bp + o_uisr_funcflag],ax
	mov	ax,#UNDI_ISR
	call	callpxe			! get next buffer from UNDI
	or	ax,ax
	jz	dohw2

	mov	word ptr [bp + o_uisr_funcflag],#UNDI_ISR_IN_FINISH
	mov	ax,#UNDI_ISR
	call	callpxe			! in case of error terminate interrupt
	jmp	dohw8			! processing

dohw2:	mov	ax,[bp + o_uisr_funcflag]
	cmp	ax,#UNDI_ISR_OUT_BUSY
	stc				! check for recursive calling
	je	dohw9
	cmp	ax,#UNDI_ISR_OUT_RECEIVE
	jne	dohw3

	mov	cx,[bp + o_uisr_headerlength]
	mov	ax,[bp + o_uisr_framelength]
	sub	ax,cx
	jbe	dohw4
	push	ax			! push frame length
	mov	ax,[bp + o_uisr_buflength]
	sub	ax,cx
	ja	dohwA
	xor	ax,ax
dohwA:	mov	[bufsize],ax		! save buffer length
#ifdef IS386
	mov	eax,[bp + o_uisr_frame]
	add	ax,cx
	mov	[bufaddr],eax
#else
	mov	ax,word ptr [bp + o_uisr_frame + 0]
	add	ax,cx
	mov	word ptr [bufaddr + 0],ax
	mov	ax,word ptr [bp + o_uisr_frame + 2]
	mov	word ptr [bufaddr + 2],ax
#endif
	mov	al,[bp + o_uisr_prottype]
	cmp	al,#UNDI_PROT_IP
	jne	dohwB
	call	_udp_recv		! handle UDP packet receive
	jmp	dohwC
dohwB:	cmp	al,#UNDI_PROT_ARP
	jne	dohwC
	call	_arp_recv		! handle ARP packet receive
dohwC:	add	sp,#2			! cleanup stack
	jmp	dohw4

dohw3:	cmp	ax,#UNDI_ISR_OUT_DONE
	je	dohw5			! check if we are done
dohw4:	mov	ax,#UNDI_ISR_IN_RECYCLE	! if not, recycle the receive buffer
	jmp	dohw1

dohw5:	call	_arp_poll		! process any packets to transmit
dohw8:	clc				! return without error
dohw9:	lahf
	add	sp,#isr_size
	sahf
	ret


!
!**************************************************************************
!
	end

