!
! first.inc  -  constants for assembler module
!
! Copyright (C) 1995-1999 Gero Kuhlmann   <gero@gkminix.han.de>
! Copyright (C) 1996,1997 Gero Kuhlmann   <gero@gkminix.han.de>
!                and Markus Gutschke <gutschk@math.uni-muenster.de>
!
!  This program is free software; you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation; either version 2 of the License, or
!  any later version.
!
!  This program is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with this program; if not, write to the Free Software
!  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
!
!====================================================================
!
! Definitions for allocating dynamic memory, which is used to hold
! temporary working buffers.
!
! The MAX_MEM segment address must be within the range of addresses
! contained in MIN_MEM number of kB.
!
MIN_MEM		equ	616		! Minimum number of kB of base mem
MAX_MEM		equ	$8000		! Maximum usable segment (see SPEC.DOC)

LINUX_HEAP	equ	1024		! Size of linux setup heap
SECTSIZE	equ	512		! Size of one block of setup code

STACK_SIZE	equ	4096		! Size of static stack memory
CMDL_SIZE	equ	2048		! Size of static command line memory

MAX_PATH_LEN	equ	1024		! Maximum length of path name
MAX_BOOTP_LEN	equ	2048		! Maximum length of bootp record
MAX_ADDRS_LEN	equ	 512		! Maximum length of IP addr string

MAX_DYNAMIC_LEN	equ	(MAX_PATH_LEN + MAX_BOOTP_LEN + MAX_ADDRS_LEN)

KERNEL_LDADDR	equ	$00100000	! kernel load address



!
!====================================================================
!
! Vendor information for the boot rom image. These values have to be
! identical to those in mknbi.h.
!
#define VENDOR_MAGIC	"GK-mknbi-linux" ! vendor ID
#define VENDOR_SIZE	4		 ! size of vendor ID in dwords

VENDOR_BOOTL	equ	16		! tag for boot loader segment
VENDOR_CMDL	equ	17		! tag for command line segment
VENDOR_INIT	equ	18		! tag for floppy boot sector
VENDOR_SETUP	equ	19		! tag for kernel setup segment
VENDOR_KERNEL	equ	20		! tag for kernel segment
VENDOR_RAMDISK	equ	21		! tag for ramdisk image



!
!====================================================================
!
! Layout and magic ID of boot rom image header. These values have to
! be identical to those used by the boot rom. See SPEC.DOC of the
! boot rom source for further information.
!
BOOT_MAGIC	equ	$1B031336	! boot image header magic cookie
BOOT_SIZE	equ	512		! total size of boot image header

BOOT_HD_SIZE	equ	 4		! size of header in dwords
BOOT_HD_MAGIC	equ	 0		! offset for header magic number
BOOT_HD_LENGTH	equ	 4		! offset for header length
BOOT_HD_FLAG1	equ	 5		! offset for header flag 1
BOOT_HD_FLAG2	equ	 6		! offset for header flag 2
BOOT_HD_FLAG3	equ	 7		! offset for header flag 3
BOOT_HD_LOCN	equ	 8		! offset for header location
BOOT_HD_EXEC	equ	12		! offset for execute address
BOOT_HD_VENDOR	equ	16		! offset for header vendor information

BOOT_LD_SIZE	equ	 4		! size of load record in dwords
BOOT_LD_LENGTH	equ	 0		! offset for load record length
BOOT_LD_TAG1	equ	 1		! offset for load record tag 1
BOOT_LD_TAG2	equ	 2		! offset for load record tag 2
BOOT_LD_FLAGS	equ	 3		! offset for load record flags
BOOT_LD_ADDR	equ	 4		! offset for absolute address
BOOT_LD_ILENGTH	equ	 8		! offset for image length
BOOT_LD_MLENGTH	equ	12		! offset for memory length
BOOT_LD_VENDOR	equ	16		! offset for vendor information

BOOT_FLAG_B0	equ	%00000001	! mask for load record flag B0
BOOT_FLAG_B1	equ	%00000010	! mask for load record flag B1
BOOT_FLAG_EOF	equ	%00000100	! mask for load record flag EOF



!
!====================================================================
!
! Ramdisk images can be loaded in three different ways:
!  RD_AUTO  --  The ramdisk image is loaded at either $100000 or right
!               behind the kernel image; at run-time it will be moved
!               to the end of physical memory or right before $1000000.
!               This technique guarantees maximum compatibility with
!               broken BIOS versions without making assumptions on the
!               way, the BOOT-Prom determined the end of memory. The
!               drawback is, that moving the image costs some extra time.
!  RD_EOM   --  The ramdisk image is loaded right before the end of
!               physical memory. It is the BOOT-Proms responsibility to do
!               this reliably. This might cause problems with some ancient
!               BIOS versions, if the computer is equipped with more then
!               16M of main memory.
!  RD_FIXED --  The user provided a fixed load address for the ramdisk
!               image. This entails possible compatibility restrictions
!               when moving the image from one client to another. OTOH,
!               it could be the final solution if everything else fails...
!
RD_AUTO		equ	0
RD_EOM		equ	1
RD_FIXED	equ	2



!
!====================================================================
!
! Layout and magic ID of bootp record. Refer to RFC 951, RFC 1048
! and RFC 1533 for further information. Vendor tag 129 is special in
! that it is in the range of tags which can be freely used (see
! RFC 1533). It will be used by first.S to add a string at the end
! of the kernel command line. The BOOTP record can be longer than
! the standard length as the bootrom might be able to load a vendor
! extension file via tftp.
!
BOOTP_MAGIC_RFC	equ	$63538263	! RFC 1048 vendor ID
BOOTP_MAGIC_LEN	equ	4		! length of vendor ID

BOOTP_REQUEST	equ	1		! bootp OP-code
BOOTP_REPLY	equ	2

BOOTP_OP	equ	  0		! offset to bootp OP-code
BOOTP_HTYPE	equ	  1		! offset to hardware type
BOOTP_HLEN	equ	  2		! offset to length of hardware addr
BOOTP_HOPS	equ	  3		! offset to number of gateway hops
BOOTP_XID	equ	  4		! offset to transfer ID
BOOTP_SECS	equ	  8		! offset to seconds since boot began
BOOTP_CIADDR	equ	 12		! offset to client IP address
BOOTP_YIADDR	equ	 16		! offset to "your" IP address
BOOTP_SIADDR	equ	 20		! offset to server IP address 
BOOTP_GIADDR	equ	 24		! offset to gateway IP address
BOOTP_CHADDR	equ	 28		! offset to hardware address
BOOTP_SNAME	equ	 44		! offset to server name
BOOTP_FILE	equ	108		! offset to name of boot file
BOOTP_VEND	equ	236		! offset to vendor information

BOOTP_SIZE	equ	300		! size of complete bootp record

BOOTP_RFC_NOP	equ	  0		! RFC vendor tag for NO-OP
BOOTP_RFC_MSK	equ	  1		! RFC vendor tag for netmask
BOOTP_RFC_GWY	equ	  3		! RFC vendor tag for gateway
BOOTP_RFC_HNAME	equ	 12		! RFC vendor tag for host name
BOOTP_RFC_ROOT	equ	 17		! RFC vendor tag for root directory
BOOTP_RFC_OVRLD	equ	 52		! RFC vendor tag for option overload
BOOTP_RFC_FILE	equ	 67		! RFC vendor tag for boot image file
BOOTP_RFC_VID	equ	128		! Vendor tag: magic identification
#ifndef BOOTP_RFC_CMDAD
BOOTP_RFC_CMDAD	equ	129		! Addition to kernel command line
#endif
#ifndef BOOTP_RFC_RDEV
BOOTP_RFC_RDEV	equ	130		! Device by which to find root FS
#endif
BOOTP_RFC_VSEL	equ	176		! index to user selected image
BOOTP_RFC_END	equ	255		! RFC vendor tag for end of record

BOOTP_VID_ID	equ	$687445E4

BOOTP_OVR_FILE	equ	%00000001
BOOTP_OVR_SNAME	equ	%00000010



!
!====================================================================
!
! The command line descriptor is placed in the floppy boot sector area
! and looks like this:
!
! $90020-$90021     2 bytes   command line magic number
! $90022-$90023     2 bytes   command line offs. relative to floppy boot sec
! $901FA-$901FB     2 bytes   video mode
!
CL_MAGIC_ADDR	equ	$20		! command line magic number
CL_MAGIC	equ	$A33F		! very unusual command sequence
CL_OFFSET	equ	$22		! command line offset
VGA_OFFSET	equ	$1FA		! video mode



!
!====================================================================
!
! The setup-header has to be changed in order to tell the kernel about a
! loaded ramdisk image
!
! +2                  4 bytes   setup magic number
! +6                  2 bytes   setup version number
! +16                 1 byte    type of loader
! +17                 1 byte    load flags
!                               bit 0: loaded high
!                               bit 7: can use heap
! +18                 2 bytes   setup move size
! +20                 4 bytes   protected mode start address
! +24                 4 bytes   ramdisk image start address
! +28                 4 bytes   ramdisk image size
! +32                 4 bytes   floppy loader helper routine pointer
! +36                 2 bytes   heap pointer
!
! This only works with Linux kernels 1.3.73 or above; that is why the
! setup magic number and the setup version number need to be verified
! before relying on the layout of this data structure.
!
SU_MAGIC_ADDR		equ	2
SU_VERSION_ADDR		equ	6
SU_TYPE_OF_LOADER	equ	16
SU_LOAD_FLAGS		equ	17
SU_MOVE_SIZE		equ	18
SU_32_START		equ	20
SU_RAMDISK_START	equ	24
SU_RAMDISK_SIZE		equ	28
SU_HEAP			equ	36

SU_MAGIC		equ	$53726448
SU_VERSION		equ	$0201
SU_MY_TYPE_OF_LOADER	equ	$41
SU_MY_LOAD_FLAGS	equ	$81

